# Filters added to this controller will be run for all controllers in the application.
# Likewise, all the methods added will be available for all controllers.

require 'datenbank'
require 'eigenschaft'
require 'liste'
require 'save_me'

class ApplicationController < ActionController::Base

  layout      "firstrails"

  include Treelist_Modul    # muss auf Klassenebene integriert werden          
  helper_method             :treelist_show, :dbs, :listlist, :do_error?, :leichen,                           
                            :test_temp

  attr_reader   :lists, :info
  attr_accessor :search
    
  # <b>initialize</b> initialisiert die Layout-Vorgaben
  
  def initialize
    @css          = "stylesheet_link_tag 'main'"
    @logo         = "image_tag(\"bussole-LogoUndWortmarke.gif\", :alt => \"bussole Logo\", :size => \"390x50\")"
    @title        = "Dateiinformationen"
    view_tree
    @current_db   = Datenbank.current_db
  end
  
  # <b>index</b> - Standardanzeige
 
  def index
    @info = params[:info] if params[:info]
    testest
  end
   
  
  # <b>message</b> - Fehler- und Meldungsausgabe
  
  def message
    @info = params[:info] if params[:info]
  end

  # <b>View_Name</b> passt die Steuervariable @view_name der aktuell sichtbaren Anzeige an
  # Anschauungsbeispiel fr Javascript-Datenbergabe beim Blttern
  
  def ViewName
    @view_name  = params[:view]
    render :nothing => true  
  end
     
  # <b>listlist</b> listet Listen auf
  
  def listlist(*list)
    liste   = Liste.new(list.flatten)
    @lists  = liste.liste
    @key    = liste.key
    @desc   = liste.desc    
    liste.select
  end
     
  # <b>listtext</b> gibt den Text zurck (Anschauungsbeispiel fr direkten Controller-Zugriff im Template)
  
  def listtext(*list)
    listlist(list.flatten)
    @desc
  end

  # <b>leichen</b> prft generell auf Leichen, unabhngig vom Dateisystem

  def leichen(*id)
    if id.empty?
      list  = [Feld.find_by_datei(@@gone)]        # ein Satz gengt zur Anzeige
      list.clear      if list[0].nil?
    else
      id    = id[0].to_i                          # muss > 0 sein
      list  = Feld.find(:all, :conditions => ["redundant = ? OR vorgaenger = ? OR nachfolger = ?", id, id, id] )
      list.insert(0, Feld.new(:name => 'in:'))    if list.length > 0     # Workaround: ist nur eine Zeile in der Liste, reagiert der observe_fields nicht mehr (keine nderung mglich)
    end
    list
  end
  
  # <b>reload_tree</b> 
  
  def reload_tree
    if params[:target] == "NaviSeite-2"
      view_tree("Eigen", "", "clear")
    else
      view_tree("noDB", "", "clear")
    end
    render :partial => "application/treelist",  :controller => "application", :locals => { :name => @view_name}
  end

  # <b>dbs</b> fordert die Datenbanken des Servers an
  
  def dbs
    Datenbank.current_db    
  end
  
  # <b>database</b> fordert die bersicht der Datenbanken sowie ggf. ihre Aktualisierung an
      
  def database
    clear = params[:update] ? 'clear_all' : 'clear'
    
    view_tree(params[:conn], params[:update], clear)
    if params[:update] && !@updated.nil? && @updated > 0
      Feld.refresh_fields(params[:conn])             # berichtigt auch Dateien
      Eigenschaft.refresh_verwendung
      view_tree("noDB", "", "clear")
    end
    render :partial => "application/nav", :controller => "application"
  end
  
  # <b>eigenschaften</b> rechnet die Eigenschaften neu durch
  
  def eigenschaften
    Eigenschaft.refresh_verwendung
    render :partial => "application/nav", :controller => "application"
  end  
  
  
  ### Testfunktionen

  # <b>test_temp</b> dient zum Debuggen von Templates

  def test_temp(*parm)
    xxx = parm[0]
  end

  # <b>test</b> dient nur zum Test, auch der Dokumentation: (#tree_listmodul.database), List, #listlist

  def test
   view_tree
   
   @info = ['hinweis', 'main']
   flash[:notice] = __FILE__, '  <i>= aktuelle Source</i><br><br>', 
                      RAILS_ROOT, '   <i>= Rails-Konfiguration</i><br><br>',  
                      #ABSOLUTE_RAILS_ROOT, '   <i>= absoluter Pfad</i><br><br>',
                      RAILS_ENV, '   <i>= Umgebung</i><br><br>', 
                      RAILS_CONNECTION_ADAPTERS, '   <i>= Datenbanktypen</i><br><br>', 
                      Datei.active_connection_name, '   <i>= aktive Verbindungen</i><br><br>',
                      Datei.connection.tables, '   <i>=  alle Dateien der angebundenen Datenbank</i><br><br>',
                      " " , Datei.__id__, "  = Klassenvariable __id__"
      
    render :action => 'index'
  end 
    
  def test1
    render :inline => "<%= 'hello, ' * 3 + 'again' %>" 
  end 
   
  def test_1z
    @info = ['hinweis', 'main']
    flash[:notice] = "gerade mal eine einzige Zeile"
    render :action => 'index'
  end
  
  def testest
    abc = ''


    xxx = 1
  end
 
 ######################################################################################################   
   
  private   

  # <b>me</b> parkt (auch nicht-marshallierbare) Objekte vorher/nachher: Parken als Objekt, Abfragen als ID

  def me(mine, todo = '', cont = self.controller_name)
    todo = todo.to_i if todo.class == String && todo.to_i.to_s == todo 
    if todo.class == Fixnum
      Save_me.new([cont, mine]).saved[todo]
    elsif todo.empty?
      Save_me.new([cont, mine]).saved  
    else
      save_me           = [mine.clone]
      if todo.class == HashWithIndifferentAccess   # attributes
        mine.attributes = todo
        save_me         << mine
      else
         save_me        << todo
      end
      Save_me.new([cont, mine.id], save_me)
    end    
  end  
       
end