
class Eigenschaft < ActiveRecord::Base
  acts_as_tree :foreign_key => 'gruppeneigenschaft'

  include Models_modul
 
  # Standard-Validierungen
  
  validates_presence_of :beschreibung

 # <b>refresh_verwendung</b> prft und ggf. korrigiert die Verwendung
  
  def Eigenschaft.refresh_verwendung
    all = find(:all)
    all.each do |eigen|
        eigen.verwendung = ''
        eigen.save_with_validation      # auer fehlender Gruppeneigenschaft kein wesentlicher Fehler mglich
      end  
  end

  def gruppe(id)
    gruppeneigenschaft_ok?(id, nil)
    @gruppe
  end
  
  # <b>save</b> fhrt das Speichern durch
  
  def save(prv = 0, nocheck = false)    
    ok = nocheck ? save_without_validation : super()
    if ok
      if prv != self.gruppeneigenschaft 
        do_save(prv)
      end                      
      do_save(self.gruppeneigenschaft)
    end
  end
 
  # <b>destroy</b> fhrt das Lschen durch
  
  def destroy
    if super      # Lschung des Datensatzes ist ok
      begin
        do_save(self.gruppeneigenschaft)
      rescue
      end
    end
  end

 
  ######################################################################################################    
  
  private  
  
  # <b>calc_conseq</b arbeitet den gesamten Datensatz ab und bestimmt rechnerische Konsequenzen
  # ! Voraussetzung ist, dass die validate-Feld-Prfungen erfolgt sind
 
  def calc_conseq(attr, store, cond)
    if store
      attr['verwendung']  = do_verwendung
    end
    attr  
  end  
     
  def art_ok?(check, cond)         # alle Felder sind als Methoden bereits angelegt: Der Call darauf gibt den Feldwert zurck
    return ''     if check.to_s.empty?
    listKey = Liste.new('EigenArt', check).key
    unless listKey == self.art || self.verwendung.empty? 
      @errors.add("art", "Die Art \"#{self.art}\" kann nicht geändert werden, solange die Eigenschaft verwendet wird") 
      self.art
    else
      listKey
    end
  end
  
  def gruppeneigenschaft_ok?(check, cond)
    return 0     unless check.to_i > 0
    if self.id == check.to_i
      @errors.add("gruppeneigenschaft", "Die Eigenschaft \"#{self.name}\" darf sich nicht selbst als Gruppeneigenschaft haben") 
    else
      begin
        @gruppe = Eigenschaft.find(check.to_i)
      rescue
        @errors.add("gruppeneigenschaft", "Die Eigenschaft \"#{check.to_i}\" kann nicht als Gruppeneigenschaft dienen, da sie nicht gefunden werden konnte. 
                    Dies k&ouml;nnte daran liegen, dass Sie auf einen &uuml;berholten Stand der Anzeige gebl&auml;ttert haben
                    und l&auml;sst sich durch \"Neuladen\" des Browsers bereinigen.") 
      end
    end
    @gruppe.nil? ? 0 : @gruppe.id     # = check
  end

  # <b>do_verwendung</b> ermittelt die Felder und Eigenschaften, die die aktuelle Eigenschaft nutzen, unabhngig vom Dateisystem
  
  def do_verwendung 
    fields_and_eigens = {}
    unless self.id == 0     # neuer Satz, kann noch nicht verwendet worden sein 
      subs              = Eigenschaft.find_all_by_gruppeneigenschaft(self.id)
      fields            = Feld.find_all_by_eigenschaft(self.id)     # Verwendung interessiert unabhngig vom Dateisystem
      
      fields_and_eigens['Eigenschaft']  = get_ids(subs)   unless subs.empty?
      fields_and_eigens['Feld']         = get_ids(fields) unless fields.empty?   
    end
    fields_and_eigens.empty? ? '' : Marshal.dump(fields_and_eigens) 
  end

  # <b>get_ids</b> sammelt die Ids aus der bergebenen Liste
  
  def get_ids(list)
    array = []
    list.each {|member| array << member.id}
    array  
  end
  
  # <b>do_save</b> ermittelt fr die bergebene Eigenschaft die Verwendung und ob sie gendert wurde
  
  def do_save(id)
    if id > 0
      begin
        eigen = Eigenschaft.find(id)
        verwendung = eigen.verwendung        
        if eigen.valid? && (verwendung != eigen.verwendung)  
          eigen.save_without_validation     # valid? schon geprft
        end
      rescue
      end    
    end    
  end  

end     # class
