class Feld < ActiveRecord::Base

  include Models_modul                  # bergibt auch @check und @info
  
  # <b>refresh_fields</b> prft und ggf. korrigiert die Verwendung von Felder einer vorgegebenen Datenbank
  
  def Feld.refresh_fields(db = '')
    if db.empty?
      all         = Feld.find(:all)
    else
      all         = Feld.find(:all, :conditions => "#{Feld.cond_dbs(db)}")
    end
    all.each do |feld|
      feld.valid?                       # falsche Eigenschaften, Redundanzen, Vorgnger, Nachfolger werden eliminiert
      feld.save_without_validation      
      end
    Datei.refresh_gewicht(db)
  end

  # <b>cond_dbs</b> bestimmt den SQL-Text fr die Beschrnkung der Feldsuche auf die Dateien einer Datenbank
    
  def Feld.cond_dbs(db, add = '', curr = 'no')    
    cond = ''
    dbs         = Datenbank.files(db, curr)
    dbs[1].each  {|file| cond <<  "#{file}, " unless file == 0}
    unless cond.empty?
      cond = add.upcase + ' datei IN (' + cond
      cond = cond[0, cond.length-2] + ')'
    end
  end
  
  # <b>save_me</b> fhrt das Speichern durch - Validierung erfolgte zuvor
  # Die Eigenschaften und Felder werden unabhngig vom Dateisystem korrigiert,
  # da Eigenschaften bergreifend verwendet werden knnen 
  
  def save_me(prv)
    do_datei  = []
    if save_without_validation
      if self.eigenschaft > 0               # Abgleich der Feldwerte
        felds = Feld.find_all_by_eigenschaft(self.eigenschaft)
        felds.each do |feld|
            if feld.anzahl != self.anzahl                 # aktueller Wert ist mageblich: self wird wegen Gleichheit nicht mehr angepackt
              feld.anzahl       = self.anzahl
              feld.eigengewicht = self.eigengewicht       # aus anzahl
              feld.save_without_validation                # Validierung kommt spter noch
              do_datei          << feld.datei     unless do_datei.include?(feld.datei)
            end
          end
      end
      if prv[0] != self.eigenschaft         # Korrektur der vorherigen Eigenschaft 
        @eigen.save(prv[0])
      end
      if prv != [self.eigenschaft, self.redundant, self.vorgaenger, self.nachfolger]
       Feld.refresh_fields()                # berichtigt auch Dateien
      else
        @datei.save
        do_datei.each {|datei|  Datei.find(datei).save}
      end
    end
  end

  # <b>exit?</b> prft darauf, ob das Feld ausgegeben wird
  
  def exit?
    anzahl_Dateien
    self.exitdistanz > @p
  end

  # <b>relativgewicht</b> ermittelt den Anteil des Feldes am Typgewicht
  # dazu werden die Exitdistanzen aller Felder derselben Eigenschaft aufsummiert,
  # mit der Tentakelzahl gemittelt und auf die Anzahl der Dateien normiert.
  # Ist das so ergebende Gewicht dann "niedrig", so gilt das Feld als "logisch".
  # Wenn es darber hinaus noch "akzentuierend" oder "dokumentarisch" ist, wird das Relativgewicht bercksichtigt.
  # Das Relativgewicht gibt demnach Auskunft ber die Bedeutung eines Feldes im Dateisystem.
  
  def relativgewicht
    datei_ok?(self.datei, nil)
    db    = @datei.datenbank
    felds = Feld.find(:all, :conditions => ["eigenschaft = ? AND exitdistanz > ? #{Feld.cond_dbs(db, 'and')}", self.eigenschaft, 0] )  # fr die eigenschaftsabhngige Summe zhlt Null nicht
    exit  = 0.0
    felds.each { |feld| exit += feld.exitdistanz}
    anzahl_Dateien
    exit  = exit / self.tentakelzahl / @p

    count = (self.typfeld ==  '+-' || self.typfeld == '-+' )
    count = count && exit <= grenze
     
    gew   = self.eigengewicht * self.profilgewicht
    
    count == true ? (return gew) : (return 0)    
  end
 
  ######################################################################################################    
  
  private  
 
  # <b>calc_conseq</b arbeitet den gesamten Datensatz ab und bestimmt rechnerische Konsequenzen
  # ! Voraussetzung ist, dass die validate-Feld-Prfungen erfolgt sind
 
  def calc_conseq(attr, store, cond)

    if store
      # bei korrekter Redundanz: nur id des charakteristischen Datenfeldes ist magebend
      if @master      
        nochange    = ['id', 'name', 'datei', 'beschreibung', 'redundant']          
        keys        = attr.keys - nochange
        keys.each {|key| attr[key] = @master.attributes[key]}
      else
        
        # Berechnungen von Distanzen und Gewichten
        anzahl_Dateien
        attr['eigengewicht']                          = eigengew(attr['anzahl'])
        attr['tentakelzahl'], attr['profilgewicht']   = tentakel(@eigen.id)
        attr['portaldistanz'], attr['portalgewicht']  = distanz('vorgaenger') 
        attr['exitdistanz'], attr['exitgewicht']      = distanz('nachfolger')
        
        attr['typfeld'], attr['typaufgabe']           = typ(attr)
      end
      
      attr
    end
    
  end     # calc_conseq
  
  # Prfung der einzelnen Felder
  
  def datei_ok?(check, cond)              # alle Felder sind als Methoden bereits angelegt: Der Call darauf gibt den Feldwert zurck
    unless check == -1
      @datei = Datei.find(check)          # Datei muss einfach existieren: auer im Fall von "Leichen"
      @errors.add("datei", "Die Datei mit der Satznummer \"#{check}\" kann nicht gefunden werden") unless @datei
    end
    @datei ? @datei.id : check
  end
  
  def beschreibung_ok?(check, cond)
    check = @datei.name.capitalize + ', ' + self.name.capitalize    if check.empty?  
    check
  end
    
  def redundant_ok?(check, cond)
    related(check, 'red')
    if @master &&  self.datei != @master.datei
      @errors.add("redundant",
            "Das Feld \"#{self.name}\" befindet sich nicht in der Datei \"#{@master.datei}\"
             des als redundant angegebenen Feldes \"#{@master.name}\".")
      @master = nil
    elsif @master && @master.eigenschaft == 0
      @errors.add("redundant",
            "Das Feld \"#{@master.name}\" ist fehlerhaft, da es keine Eigenschaft aufweist.")
      @master = nil   
    end
    @master ? @master.id : 0
  end
  
  # <br>anzahl_ok?</br> prft die Anzahl der Feldwerte - Besonderheit: es werden keine Fehler ausgegeben, sondern Vorbelegungen durchgefhrt
  
  def anzahl_ok?(check, eigen = self.eigenschaft)
    if eigen.to_i > 0 && check.to_i == 0
        feld    = Feld.find(:first, :conditions => ["eigenschaft = ? AND anzahl > ? AND id <> ?", eigen.to_i, 0, self.id] )    # erster passender Satz sollte gengen, da diesser Wert eigenschaftsabhngig ist und deshalb in allen Dateisystemen gleich sein sollte
        if feld.nil?
          check = 0
        else
          check = feld.anzahl
          msg   = "F&uuml;r die Anzahl der Feldwerte wurde der Wert von 
                  Feld \"#{feld.name}\" aus der Datei \"#{Datei.find(feld.datei).name}\" mit \"#{feld.anzahl}\" angesetzt, 
                  da es denselben Inhalt speichert." 
          info("anzahl", msg)
        end
    end
    if check.to_i == 0
      if @datei
        check = @datei.records
      else
        check = 101         # damit Eigengewicht "rund" wird    
      end
      if @check && @check.length == 1     # interaktive Prfung
        if self.anzahl > 0
        check = self.anzahl
          msg = "F&uuml;r die Anzahl der Feldwerte wird ein Wert von \"#{check}\" angesetzt." 
        else
          msg = "Die angegebene Anzahl der Werte dieses Felds kann nicht Null sein.<br>
                  Wenn die genaue Menge der Werte nicht bekannt ist, gen&uuml;gt eine gute Sch&auml;tzung.<br>
                  Vorl&auml;ufig wird ein Wert von \"#{check}\" angesetzt."
        end
        info('anzahl', msg)
      end

    elsif eigen.to_i > 0                # Anzahl muss bei allen Feldern derselben Eigenschaft gleich sein (dateibergreifend)
      feld    = Feld.find(:first, :conditions => ["eigenschaft = ? AND anzahl <> ? AND id <> ?", eigen.to_i, check.to_i, self.id] )    # erster passender Satz sollte gengen
      unless feld.nil?
        msg   = ""
        msg   = "<br>"      unless @errors['anzahl'].nil?     # sollte gar nicht vorkommen knnen
        msg   << "Die angegebene Anzahl \"#{check}\" stimmt mit bisherigen Feldwerten derselben Eigenschaft nicht &uuml;berein.<br>
                F&uuml;r das Feld \"#{feld.name}\" aus der Datei \"#{feld.datei}\" liegt beispielsweise ein Wert von \"#{feld.anzahl}\" vor. " + '<br>'
        if eigen.to_i  == self.eigenschaft 
          msg << 'Wenn Sie Ihre Eingabe nicht zu &auml;ndern w&uuml;nschen, wird die gew&auml;hlte Anzahl auch in die &uuml;brigen Felder &uuml;bernommen'
        else
          msg << 'Bei dem angeforderten Eigenschaftswechsel wird die bisherige Eingabe der Anzahl ignoriert.'
          check = feld.anzahl
        end
        info('anzahl', msg)
      end        
    end      
    check
  end

  def eigenschaft_ok?(check, cond)
    begin
    @eigen = Eigenschaft.find(check.to_i)   if check.to_i > 0  
      unless @eigen
        do_eigen(":all, :conditions => \"name = '#{self.name}'\" ")
      end
      unless @eigen
        do_eigen(":all, :conditions => \"name like '%#{self.name}'\" ")
      end
      unless @eigen
        do_eigen(":all, :conditions => \"name like '%#{self.name}%'\" ")
      end
      if !@eigen && self.name.include?('_')
        str = self.name[0, self.name.index('_')]        
        do_eigen(":all, :conditions => \"name = '#{str}'\" " )
      end 
    rescue
    end
      
    # immer noch nichts gefunden
    msg = check.to_i > 0 ? "Die Eigenschaft \"#{check}\" existiert nicht" : "Es liegt keine Eigenschaft vor - sie wird aber gebraucht"
    if !@eigen
      @errors.add("eigenschaft", msg) 
    elsif @eigen.id != self.eigenschaft
      # Anzahl bercksichtigen bei Eigenschaftsnderung
      self.anzahl = anzahl_ok?(0, @eigen.id)          
      info('eigenschaft', info('anzahl'))     # die Nachricht der Eigenschaft "zuschlagen"  
      info('anzahl',nil,'-')
    end
        
    @eigen ? @eigen.id : 0    
  end  
  
  def vorgaenger_ok?(check, cond)
    related(check, 'vor')
    if @vor &&  self.datei == @vor.datei
      @errors.add("vorgaenger",
            "Das Feld \"#{self.name}\" befindet sich in derselben Datei \"#{@vor.datei}\"
             wie das  als Vorg&auml;nger angegebene Feld \"#{@vor.name}\".")
      @vor = nil
    end
    @vor ? @vor.id : 0
  end
  
  def nachfolger_ok?(check, cond)
    if check.to_i < 0      # Endstation - keine Ausgabe vorgesehen
      @nach = Feld.new(:name => "Endstation", :nachfolger => 0)
      @nach.id = -1
    else
      related(check, 'nach')
      if @nach &&  self.datei == @nach.datei
        @errors.add("nachfolger",
              "Das Feld \"#{self.name}\" befindet sich in derselben Datei \"#{@nach.datei}\"
               wie das als Nachfolger angegebene Feld \"#{@nach.name}\".")
        @nach = nil
      end
    end
    @nach ? @nach.id : 0
  end
  
  # Berechnungen von Gewichten und Distanzen
  
  def anzahl_Dateien    
    datei_ok?(self.datei, nil)   unless @datei    
    if !@p && @datei
      sql = "SELECT count( * ) FROM `dateis` WHERE datenbank = '#{@datei.datenbank}'"
      list = []
      Feld.connection.execute(sql).each { |row| list = row}   # nur ein Ergebnis
      @p = list[0].to_i
    end
    @p = 1      if !@p || @p == 0
  end
  
  def eigengew(check)
    check > 1 ? 1.0/(check-1) : 1.0
  end
  
  # <b>tentakel</b> bestimmt die Verwendung der Eigenschaft - @eigen.verwendung nicht benutzt, um Abstimmungsproblemen zu entkommen
  # Die Tentakelzahl ist auf das Dateisystem beschrnkt, da sie Auskunft ber die "Wichtigkeit" eines Inhaltes fr das System gibt
  
  def tentakel(check)    
    db    = @datei.datenbank
    files = check > 0 ? Feld.find(:all, :conditions => ["eigenschaft = ? AND redundant = ? #{Feld.cond_dbs(db, 'and')}", check, 0], :order => "datei" ) : []  # wenigstens die eigene Eigenschaft muss vorliegen, wobei keine Redundanz vorliegen darf
    # Vorkommen pro Datei, nicht pro Feld
    unless files[0].nil?
      count, datei  = 0.0, 0
      files.each do |feld|
        count       += 1.0  unless datei == feld.datei
        datei       = feld.datei        
        end
    else
      count         = 1.0
    end      
    return count, count/@p
  end  
   
  def distanz(feldname)
    id      = 0
    # Vorlufer und Nachfolger sind bereits geprft
    if    @vor   && feldname == 'vorgaenger'
      id    = @vor.id
      check = @vor.vorgaenger
    elsif @nach  && feldname == 'nachfolger'
      id    = @nach.id
      check = @nach.nachfolger
    end
    if id == 0 || nil
      return 0,0
    elsif id == -1      # Endstation
      return @p+1, (@p+1).to_f/@p
    else
      return weiter(check, feldname, 1, [self.id, id ])
    end
  end

  def weiter(check, feldname, lauf, list)    
    if check > 0 && !list.include?(check)
      feld    = Feld.find(check)
      if feld.id > 0 
        lauf    += 1
        check   = eval('feld.' + feldname)
        lauf, x = weiter(check, feldname, lauf, list << feld.id)   if check > 0
      end
    end    
    return lauf, lauf.to_f/@p
  end
  
  def typ(attr)
    limit = grenze
    ge    = attr['eigengewicht']   > limit*0.7  ? '+' : '-'      # Eigengewicht ist schnell zu klein
    gf    = attr['profilgewicht']  > limit      ? '+' : '-'
    gp    = attr['portalgewicht']  > limit      ? '+' : '-'
    gx    = attr['exitgewicht']    > limit      ? '+' : '-'
    return ge+gf, gp+gx    
  end
  
  def related(check, todo)
    return 0     unless check.to_i > 0
    datei_ok?(self.datei, nil)   unless @datei
    case todo
    when 'red'
      name  = 'Redundanz'
      feld  = 'redundant'
      inst  = '@master'
    when 'vor'
      name  = 'Vorg&auml;nger'
      feld  = 'vorgaenger'
      inst  = '@vor'
    when 'nach'
      name  = 'Nachfolger'
      feld  = 'nachfolger'
      inst  = '@nach'
    end
    
    if self.id == check.to_i
      @errors.add("#{feld}", "Das Feld \"#{self.name}\" darf sich nicht selbst als #{name} haben") 
    else
      begin
        related = Feld.find(check.to_i)
      rescue
        @errors.add("#{feld}", "Das Feld \"#{check.to_i}\" kann nicht als #{name} dienen, da es nicht gefunden werden konnte. 
                    Dies k&ouml;nnte daran liegen, dass Sie auf einen &uuml;berholten Stand der Anzeige gebl&auml;ttert haben
                    und l&auml;sst sich durch \"Neuladen\" des Browsers bereinigen.") 
      end
    end

    if related && related.redundant > 0    
      begin
        red = Feld.find(related.redundant)
        @errors.add("#{feld}", "Das Feld \"#{related.name}\" kann nicht als #{name} dienen, da es nur ein redundantes Feld ist. 
                Das f&uuml;hrende Feld ist \"#{red.name}\" ")
      rescue
        @errors.add("#{feld}", "Das Feld \"#{related.name}\" kann nicht als #{name} dienen, da es nur ein redundantes Feld ist. 
                Das f&uuml;hrende Feld ist als \"#{related.redundant}\" angegeben, konnte jedoch nicht gefunden werden. ")        
      
      end 
      related = nil
    
    elsif related
      datei = Datei.find(related.datei)
      unless datei.datenbank == @datei.datenbank
        @errors.add("#{feld}", "Das Feld \"#{related.name}\" kann nicht als #{name} dienen, da es nicht aus demselben Dateisystem stammt.")       
        related = nil      
      end
    end
    eval("#{inst} = related")
  end

  def do_eigen(cond)
    list = eval("Eigenschaft.find(#{cond})")
    if list.length == 1
      @eigen = list[0]
    elsif list.length > 1
      # Dateiname in der Eigenschaft enthalten
      cand = list.select { |rec| rec.attributes['name'].include?(@datei.name) }
      cand = list.select { |rec| rec.attributes['name'].include?(@datei.name[0,4]) } if cand.length == 0
      cand = list.select { |rec| rec.attributes['name'].upcase.include?(@datei.name[0,4].upcase) } if cand.length == 0
      @eigen = cand[0]  if cand.length == 1
    end
  end
  
  def grenze
    1.0/@p
  end
  
end
